@@ -36,6 +36,8 @@ pub enum Error {
     DbcMintRequestDoesNotBalance { input: u64, output: u64 },
     #[error("Outputs must be numbered 0..N where N = # of outputs")]
     OutputsAreNotNumberedCorrectly,
+    #[error("Failed to unblind an input DBC")]
+    FailedUnblinding,
     #[error("DBC already spent in transaction: {transaction:?}")]
     DbcAlreadySpent {
         transaction: crate::DbcTransaction,
